@@ -19,10 +19,8 @@ package org.apache.commons.jexl3;
 import org.apache.commons.jexl3.internal.Debugger;
 import org.apache.commons.jexl3.internal.TemplateDebugger;
 import org.apache.commons.jexl3.internal.TemplateInterpreter;
-import org.apache.commons.jexl3.internal.TemplateScript;
 import org.apache.commons.jexl3.internal.introspection.Permissions;
 import org.apache.commons.jexl3.internal.introspection.Uberspect;
-import org.apache.commons.jexl3.introspection.JexlMethod;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
@@ -35,8 +33,6 @@ import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
 import java.util.Set;
-import java.util.logging.Logger;
-
 import org.junit.After;
 import org.junit.Assert;
 import org.junit.Before;
@@ -69,11 +65,10 @@ public class JXLTTest extends JexlTestCase {
    public static List<JexlBuilder> engines() {
        final JexlFeatures f = new JexlFeatures();
        f.lexical(true).lexicalShade(true);
-      return Arrays.<JexlBuilder>asList(new JexlBuilder().silent(false)
-        .lexical(true).lexicalShade(true)
-        .cache(128).strict(true), new JexlBuilder().features(f).silent(false)
-        .cache(128).strict(true), new JexlBuilder().silent(false)
-        .cache(128).strict(true));
+      return Arrays.<JexlBuilder>asList(
+              new JexlBuilder().silent(false).lexical(true).lexicalShade(true).cache(128).strict(true),
+              new JexlBuilder().features(f).silent(false).cache(128).strict(true),
+              new JexlBuilder().silent(false).cache(128).strict(true));
    }
 
     @Before
@@ -268,7 +263,7 @@ public class JXLTTest extends JexlTestCase {
     }
 
     @Test
-    public void testConstant() throws Exception {
+    public void testConstant0() throws Exception {
         final JexlContext none = null;
         final String source = "Hello World!";
         final JxltEngine.Expression expr = JXLT.createExpression(source);
@@ -783,12 +778,31 @@ public class JXLTTest extends JexlTestCase {
     @Test
     public void testInterpolationParameter() throws Exception {
         final String expr =  "(user)->{`Hello \n${user}`}";
-        Object value = ENGINE.createScript(expr).execute(context, "Henrib");
+        JexlScript script = ENGINE.createScript(expr);
+        Object value = script.execute(context, "Henrib");
         Assert.assertEquals(expr, "Hello \nHenrib", value);
         value = ENGINE.createScript(expr).execute(context, "Dimitri");
         Assert.assertEquals(expr, "Hello \nDimitri", value);
     }
 
+    @Test
+    public void testDbgEscapes() throws Exception {
+        String[] srcs = new String[]{
+                "jexl:print('hello\\'\\nworld')",
+                "'hello\\tworld'",
+                "'hello\\nworld'",
+                "'hello\\fworld'",
+                "'hello\\rworld'"
+        };
+        for(String src : srcs) {
+            JexlScript script = ENGINE.createScript(src);
+            Debugger dbg = new Debugger();
+            dbg.debug(script);
+            String msrc = dbg.toString();
+            Assert.assertEquals(src, msrc);
+        }
+    }
+
     @Test
     public void testImmediateTemplate() throws Exception {
         context.set("tables", new String[]{"table1", "table2"});
@@ -1002,40 +1016,6 @@ public class JXLTTest extends JexlTestCase {
         Assert.assertEquals(s315, output);
     }
 
-        // define mode pro50
-    static final JexlOptions MODE_PRO50 = new JexlOptions();
-    static {
-        MODE_PRO50.setFlags( "+strict +cancellable +lexical +lexicalShade -safe".split(" "));
-    }
-
-    public static class PragmaticContext extends MapContext implements JexlContext.PragmaProcessor, JexlContext.OptionsHandle {
-        private final JexlOptions options;
-
-        public PragmaticContext(final JexlOptions o) {
-            this.options = o;
-        }
-
-        @Override
-        public void processPragma(final String key, final Object value) {
-            if ("script.mode".equals(key) && "pro50".equals(value)) {
-                options.set(MODE_PRO50);
-            }
-        }
-
-        @Override
-        public Object get(final String name) {
-            if ("$options".equals(name)) {
-                return options;
-            }
-            return super.get(name);
-        }
-
-        @Override
-        public JexlOptions getEngineOptions() {
-            return options;
-        }
-    }
-
     @Test
     public void testLexicalTemplate() throws Exception {
         final JexlOptions opts = new JexlOptions();
@@ -1055,7 +1035,12 @@ public class JXLTTest extends JexlTestCase {
         final Writer strw0 = new StringWriter();
         tmplt0.evaluate(ctxt, strw0);
         final String output0 = strw0.toString();
-        Assert.assertEquals( "-strict -cancellable -lexical -lexicalShade +safe", output0);
+        JexlFeatures features = BUILDER.features();
+        if (features != null && features.isLexical() && features.isLexicalShade()) {
+            Assert.assertEquals("-strict -cancellable +lexical +lexicalShade +safe", output0);
+        } else {
+            Assert.assertEquals("-strict -cancellable -lexical -lexicalShade +safe", output0);
+        }
 
         final String src = "$$ #pragma script.mode pro50\n" + src0;
 
