@@ -20,7 +20,6 @@ import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Calendar;
-import java.util.Collections;
 import java.util.Date;
 import java.util.HashSet;
 import java.util.LinkedHashMap;
@@ -28,11 +27,12 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.TimeZone;
-import org.apache.commons.jexl3.parser.JexlParser;
-import org.junit.Assert;
-import org.junit.Test;
+
+import org.apache.commons.jexl3.parser.StringParser;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+import org.junit.Assert;
+import org.junit.Test;
 
 /**
  * Tests local variables.
@@ -167,7 +167,7 @@ public class VarTest extends JexlTestCase {
      * @return the set of variables
      */
     Set<List<String>> mkref(final String[][] refs) {
-        final Set<List<String>> set = new HashSet<List<String>>();
+        final Set<List<String>> set = new HashSet<>();
         for(final String[] ref : refs) {
             set.add(Arrays.asList(ref));
         }
@@ -197,7 +197,7 @@ public class VarTest extends JexlTestCase {
     }
 
     List<String> stringify(final Set<List<String>> sls) {
-        final List<String> ls = new ArrayList<String>();
+        final List<String> ls = new ArrayList<>();
         for(final List<String> l : sls) {
         final StringBuilder strb = new StringBuilder();
         for(final String s : l) {
@@ -206,7 +206,7 @@ public class VarTest extends JexlTestCase {
         }
             ls.add(strb.toString());
         }
-        Collections.sort(ls);
+        ls.sort(null);
         return ls;
     }
 
@@ -426,7 +426,7 @@ public class VarTest extends JexlTestCase {
          * @return the property values
          */
         public List<String> get(final List<String> keys) {
-            final List<String> values = new ArrayList<String>();
+            final List<String> values = new ArrayList<>();
             for(final String key : keys) {
                 final String value = get(key);
                 if (value != null) {
@@ -444,7 +444,7 @@ public class VarTest extends JexlTestCase {
          * @return the alia map
          */
         public Map<String,Object> get(final Map<String,String> map) {
-            final Map<String,Object> values = new LinkedHashMap<String,Object>();
+            final Map<String,Object> values = new LinkedHashMap<>();
             for(final Map.Entry<String,String> entry : map.entrySet()) {
                 final String value = get(entry.getKey());
                 if (value != null) {
@@ -461,7 +461,7 @@ public class VarTest extends JexlTestCase {
      * @return the properties array
      */
     private static String[] readIdentifiers(final String str) {
-        final List<String> ids = new ArrayList<String>();
+        final List<String> ids = new ArrayList<>();
         StringBuilder strb = null;
         String id = null;
         char kind = 0; // array, set or map kind using first char
@@ -485,7 +485,7 @@ public class VarTest extends JexlTestCase {
             }
             else if (c == '\'' || c == '"') {
                 strb.append(c);
-                final int l = JexlParser.readString(strb, str, i + 1, c);
+                final int l = StringParser.readString(strb, str, i + 1, c);
                 if (l > 0) {
                     id = strb.substring(1, strb.length() - 1);
                     strb.delete(0, l + 1);
@@ -494,7 +494,7 @@ public class VarTest extends JexlTestCase {
             }
             // discard all chars not in identifier
         }
-        return ids.toArray(new String[ids.size()]);
+        return ids.toArray(new String[0]);
     }
 
     @Test
@@ -616,7 +616,7 @@ public class VarTest extends JexlTestCase {
     @Test
     public void testObjectContext() throws Exception {
         final TheVarContext vars = new TheVarContext();
-        final JexlContext jc = new ObjectContext<TheVarContext>(JEXL, vars);
+        final JexlContext jc = new ObjectContext<>(JEXL, vars);
         try {
             JexlScript script;
             Object result;
