@@ -17,11 +17,16 @@
 package org.apache.commons.jexl3.introspection;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.List;
+import java.util.Map;
+
+import org.apache.commons.jexl3.JexlArithmetic;
 import org.apache.commons.jexl3.JexlBuilder;
 import org.apache.commons.jexl3.JexlContext;
 import org.apache.commons.jexl3.JexlEngine;
 import org.apache.commons.jexl3.JexlException;
+import org.apache.commons.jexl3.JexlExpression;
 import org.apache.commons.jexl3.JexlScript;
 import org.apache.commons.jexl3.JexlTestCase;
 import org.apache.commons.jexl3.MapContext;
@@ -139,7 +144,7 @@ public class SandboxTest extends JexlTestCase {
             Assert.fail("ctor should not be accessible");
         } catch (final JexlException.Method xmethod) {
             // ok, ctor should not have been accessible
-            LOGGER.info(xmethod.toString());
+            LOGGER.debug(xmethod.toString());
         }
     }
 
@@ -162,7 +167,7 @@ public class SandboxTest extends JexlTestCase {
             Assert.fail("Quux should not be accessible");
         } catch (final JexlException.Method xmethod) {
             // ok, Quux should not have been accessible
-            LOGGER.info(xmethod.toString());
+            LOGGER.debug(xmethod.toString());
         }
     }
 
@@ -185,7 +190,7 @@ public class SandboxTest extends JexlTestCase {
             Assert.fail("alias should not be accessible");
         } catch (final JexlException.Property xvar) {
             // ok, alias should not have been accessible
-            LOGGER.info(xvar.toString());
+            LOGGER.debug(xvar.toString());
         }
     }
 
@@ -208,7 +213,7 @@ public class SandboxTest extends JexlTestCase {
             Assert.fail("alias should not be accessible");
         } catch (final JexlException.Property xvar) {
             // ok, alias should not have been accessible
-            LOGGER.info(xvar.toString());
+            LOGGER.debug(xvar.toString());
         }
     }
 
@@ -289,7 +294,7 @@ public class SandboxTest extends JexlTestCase {
                 Assert.fail("should have not been possible");
             } catch (JexlException.Method | JexlException.Property xjm) {
                 // ok
-                LOGGER.info(xjm.toString());
+                LOGGER.debug(xjm.toString());
             }
         }
     }
@@ -342,7 +347,12 @@ public class SandboxTest extends JexlTestCase {
         // can not create a new file
         sandbox.block(java.io.File.class.getName()).execute("");
 
-        final JexlEngine sjexl = new JexlBuilder().sandbox(sandbox).safe(false).strict(true).create();
+        final JexlEngine sjexl = new JexlBuilder()
+                .permissions(JexlPermissions.UNRESTRICTED)
+                .sandbox(sandbox)
+                .safe(false)
+                .strict(true)
+                .create();
 
         String expr;
         JexlScript script;
@@ -353,7 +363,7 @@ public class SandboxTest extends JexlTestCase {
             result = script.execute(context);
             Assert.fail("should not allow calling exit!");
         } catch (final JexlException xjexl) {
-            LOGGER.info(xjexl.toString());
+            LOGGER.debug(xjexl.toString());
         }
 
         script = sjexl.createScript("System.exit(1)");
@@ -361,7 +371,7 @@ public class SandboxTest extends JexlTestCase {
             result = script.execute(context);
             Assert.fail("should not allow calling exit!");
         } catch (final JexlException xjexl) {
-            LOGGER.info(xjexl.toString());
+            LOGGER.debug(xjexl.toString());
         }
 
         script = sjexl.createScript("new('java.io.File', '/tmp/should-not-be-created')");
@@ -369,7 +379,7 @@ public class SandboxTest extends JexlTestCase {
             result = script.execute(context);
             Assert.fail("should not allow creating a file");
         } catch (final JexlException xjexl) {
-            LOGGER.info(xjexl.toString());
+            LOGGER.debug(xjexl.toString());
         }
 
         expr = "System.currentTimeMillis()";
@@ -382,7 +392,7 @@ public class SandboxTest extends JexlTestCase {
        public void testSandboxInherit0() throws Exception {
         Object result;
         final JexlContext ctxt = null;
-        final List<String> foo = new ArrayList<String>();
+        final List<String> foo = new ArrayList<>();
         final JexlSandbox sandbox = new JexlSandbox(false, true);
         sandbox.allow(java.util.List.class.getName());
 
@@ -450,10 +460,76 @@ public class SandboxTest extends JexlTestCase {
             Assert.fail("should not be possible");
         } catch (final JexlException xjm) {
             // ok
-            LOGGER.info(xjm.toString());
+            LOGGER.debug(xjm.toString());
         }
     }
+    public interface SomeInterface {
+        int bar();
+    }
 
+    public static class Foo386 implements SomeInterface {
+        @Override
+        public int bar() {
+            return 42;
+        }
+    }
+    public static class Quux386 extends Foo386 {
+        @Override
+        public int bar() {
+            return -42;
+        }
+    }
+    @Test
+    public void testInheritedPermission0() {
+        final Foo386 foo = new Foo386();
+        final JexlSandbox sandbox = new JexlSandbox(false, true);
+        sandbox.permissions(SomeInterface.class.getName(), true, true, true, true);
+        final JexlEngine sjexl = new JexlBuilder().sandbox(sandbox).safe(false).strict(true).create();
+        final JexlScript someOp = sjexl.createScript("foo.bar()", "foo");
+        Assert.assertEquals(42, someOp.execute(null, foo));
+    }
+
+    @Test
+    public void testNonInheritedPermission0() {
+        final Foo386 foo = new Foo386();
+        final JexlSandbox sandbox = new JexlSandbox(false, true);
+        sandbox.permissions(SomeInterface.class.getName(), false, true, true, true);
+        final JexlEngine sjexl = new JexlBuilder().sandbox(sandbox).safe(false).strict(true).create();
+        final JexlScript someOp = sjexl.createScript("foo.bar()", "foo");
+
+        try {
+            someOp.execute(null, foo);
+            Assert.fail("should not be possible");
+        } catch (final JexlException e) {
+            // ok
+            LOGGER.debug(e.toString());
+        }
+    }
+    @Test
+    public void testInheritedPermission1() {
+        final Quux386 foo = new Quux386();
+        final JexlSandbox sandbox = new JexlSandbox(false, true);
+        sandbox.permissions(Foo386.class.getName(), true, true, true, true);
+        final JexlEngine sjexl = new JexlBuilder().sandbox(sandbox).safe(false).strict(true).create();
+        final JexlScript someOp = sjexl.createScript("foo.bar()", "foo");
+        Assert.assertEquals(-42, someOp.execute(null, foo));
+    }
+    @Test
+    public void testNonInheritedPermission1() {
+        final Quux386 foo = new Quux386();
+        final JexlSandbox sandbox = new JexlSandbox(false, true);
+        sandbox.permissions(Foo386.class.getName(), false, true, true, true);
+        final JexlEngine sjexl = new JexlBuilder().sandbox(sandbox).safe(false).strict(true).create();
+        final JexlScript someOp = sjexl.createScript("foo.bar()", "foo");
+
+        try {
+            someOp.execute(null, foo);
+            Assert.fail("should not be possible");
+        } catch (final JexlException e) {
+            // ok
+            LOGGER.debug(e.toString());
+        }
+    }
     public static class Foo42 {
         public int getFoo() {
             return 42;
@@ -488,4 +564,137 @@ public class SandboxTest extends JexlTestCase {
             Assert.assertNotNull(xany);
         }
     }
+
+    @Test
+    public void testGetNullKeyAllowed0() throws Exception {
+        JexlEngine jexl = new JexlBuilder().sandbox(new JexlSandbox(true)).create();
+        JexlExpression expression = jexl.createExpression("{null : 'foo'}[null]");
+        Object o = expression.evaluate(null);
+        Assert.assertEquals("foo", o);
+    }
+
+    @Test
+    public void testGetNullKeyAllowed1() throws Exception {
+        JexlSandbox sandbox = new JexlSandbox(true, true);
+        JexlSandbox.Permissions p = sandbox.permissions("java.util.Map", false, true, true);
+        p.read().add("quux");
+        JexlEngine jexl = new JexlBuilder().sandbox(sandbox).create();
+        // cant read quux
+        String q = "'quux'"; //quotes are important!
+        JexlExpression expression = jexl.createExpression("{"+q+" : 'foo'}["+q+"]");
+        try {
+            Object o = expression.evaluate(null);
+            Assert.fail("should have blocked " + q);
+        } catch (JexlException.Property xp) {
+            Assert.assertTrue(xp.getMessage().contains("undefined"));
+        }
+        // can read foo, null
+        for(String k : Arrays.asList("'foo'", "null")) {
+            expression = jexl.createExpression("{"+k+" : 'foo'}["+k+"]");
+            Object o = expression.evaluate(null);
+            Assert.assertEquals("foo", o);
+        }
+    }
+
+    @Test
+    public void testGetNullKeyBlocked() throws Exception {
+        JexlSandbox sandbox = new JexlSandbox(true, true);
+        JexlSandbox.Permissions p = sandbox.permissions("java.util.Map", false, true, true);
+        p.read().add(null);
+        p.read().add("quux");
+        // can read bar
+        JexlEngine jexl = new JexlBuilder().sandbox(sandbox).create();
+        JexlExpression e0 = jexl.createExpression("{'bar' : 'foo'}['bar']");
+        Object r0 = e0.evaluate(null);
+        Assert.assertEquals("foo", r0);
+        // can not read quux, null
+        for(String k : Arrays.asList("'quux'", "null")) {
+            JexlExpression expression = jexl.createExpression("{"+k+" : 'foo'}["+k+"]");
+            try {
+                Object o = expression.evaluate(null);
+                Assert.fail("should have blocked " + k);
+            } catch (JexlException.Property xp) {
+                Assert.assertTrue(xp.getMessage().contains("undefined"));
+            }
+        }
+    }
+
+    public static class Arithmetic350 extends JexlArithmetic {
+        // cheat and keep the map builder around
+        MapBuilder mb = new org.apache.commons.jexl3.internal.MapBuilder(3);
+        public Arithmetic350(boolean astrict) {
+            super(astrict);
+        }
+        @Override
+        public MapBuilder mapBuilder(final int size) {
+            return mb;
+        }
+        Map<?,?> getLastMap() {
+            return (Map<Object,Object>) mb.create();
+        }
+    }
+
+    @Test
+    public void testSetNullKeyAllowed0() throws Exception {
+        Arithmetic350 a350 = new Arithmetic350(true);
+        JexlEngine jexl = new JexlBuilder().arithmetic(a350).sandbox(new JexlSandbox(true)).create();
+        JexlContext jc = new MapContext();
+        JexlExpression expression = jexl.createExpression("{null : 'foo'}[null] = 'bar'");
+        expression.evaluate(jc);
+        Map<?,?> map = a350.getLastMap();
+        Assert.assertEquals("bar", map.get(null));
+    }
+
+    @Test
+    public void testSetNullKeyAllowed1() throws Exception {
+        Arithmetic350 a350 = new Arithmetic350(true);
+        JexlSandbox sandbox = new JexlSandbox(true, true);
+        JexlSandbox.Permissions p = sandbox.permissions("java.util.Map", true, false, true);
+        p.write().add("quux");
+        JexlEngine jexl = new JexlBuilder().arithmetic(a350).sandbox(sandbox).create();
+        // can not write quux
+        String q = "'quux'"; //quotes are important!
+        JexlExpression expression = jexl.createExpression("{"+q+" : 'foo'}["+q+"] = '42'");
+        try {
+            Object o = expression.evaluate(null);
+            Assert.fail("should have blocked " + q);
+        } catch (JexlException.Property xp) {
+            Assert.assertTrue(xp.getMessage().contains("undefined"));
+        }
+        // can write bar, null
+        expression = jexl.createExpression("{'bar' : 'foo'}['bar'] = '42'");
+        expression.evaluate(null);
+        Map<?, ?> map = a350.getLastMap();
+        Assert.assertEquals("42", map.get("bar"));
+        map.clear();
+        expression = jexl.createExpression("{null : 'foo'}[null] = '42'");
+        expression.evaluate(null);
+        map = a350.getLastMap();
+        Assert.assertEquals("42", map.get(null));
+    }
+
+    @Test
+    public void testSetNullKeyBlocked() throws Exception {
+        Arithmetic350 a350 = new Arithmetic350(true);
+        JexlSandbox sandbox = new JexlSandbox(true, true);
+        JexlSandbox.Permissions p = sandbox.permissions("java.util.Map", true, false, true);
+        p.write().add(null);
+        p.write().add("quux");
+        JexlEngine jexl = new JexlBuilder().arithmetic(a350).sandbox(sandbox).create();
+        // can write bar
+        JexlExpression expression = jexl.createExpression("{'bar' : 'foo'}['bar'] = '42'");
+        expression.evaluate(null);
+        Map<?,?> map = a350.getLastMap();
+        Assert.assertEquals("42", map.get("bar"));
+        // can not write quux, null
+        for(String k : Arrays.asList("'quux'", "null")) {
+            expression = jexl.createExpression("{"+k+" : 'foo'}["+k+"] = '42'");
+            try {
+                Object o = expression.evaluate(null);
+                Assert.fail("should have blocked " + k);
+            } catch (JexlException.Property xp) {
+                Assert.assertTrue(xp.getMessage().contains("undefined"));
+            }
+        }
+    }
 }
